@@ -17,7 +17,7 @@ my $pa  = Perinci::Access::InProcess->new(
     extra_wrapper_args => {remove_internal_properties=>0},
 );
 
-our $VERSION = '0.02'; # VERSION
+our $VERSION = '0.03'; # VERSION
 
 use Moose;
 use namespace::autoclean;
@@ -79,6 +79,9 @@ sub munge_file {
     my $sub_has_vargs; # VALIDATE_ARGS has been declared for current sub
     my %vargs; # list of validated args for current sub, val 2=skipped
     my %vsubs; # list of subs
+    my %vars;    # list of variables that the generated validator needs
+    my @modules; # list of modules that the generated validator needs
+
     my $i = 0; # line number
 
     my $check_prev_sub = sub {
@@ -121,15 +124,25 @@ sub munge_file {
 
     my $gen_arg = sub {
         my $meta = $metas->{$sub_name};
+        my $dn = $arg; $dn =~ s/\W+/_/g;
         my $cd = $plc->compile(
             schema      => $meta->{args}{$arg}{schema},
             err_term    => '$arg_err',
-            data_name   => $arg,
+            data_name   => $dn,
             data_term   => $var,
             return_type => 'str',
             comment     => 0,
         );
         my @code;
+        for (@{$cd->{modules}}) {
+            push @code, "require $_; " unless $_ ~~ @modules;
+            push @modules, $_;
+        }
+        for (sort keys %{$cd->{vars}}) {
+            push @code, "my \$$_ = ".$plc->literal($cd->{vars}{$_})."; "
+                unless exists($vars{$_});
+            $vars{$_}++;
+        }
         push @code, 'my $arg_err; ' unless keys %vargs;
         push @code, __squish_code($cd->{result}), "; ";
         push @code, $gen_verr->('$arg_err', $arg);
@@ -154,14 +167,24 @@ sub munge_file {
             }
             my $s = $meta->{args}{$arg}{schema};
             if ($s) {
+                my $dn = $arg; $dn =~ s/\W+/_/g;
                 my $cd = $plc->compile(
                     schema      => $s,
                     err_term    => '$arg_err',
-                    data_name   => $arg,
+                    data_name   => $dn,
                     data_term   => $kvar,
                     return_type => 'str',
                     comment     => 0,
                 );
+                for (@{$cd->{modules}}) {
+                    push @code, "require $_; " unless $_ ~~ @modules;
+                    push @modules, $_;
+                }
+                for (sort keys %{$cd->{vars}}) {
+                    push @code, "my \$$_ = ".$plc->literal($cd->{vars}{$_})."; "
+                        unless exists($vars{$_});
+                    $vars{$_}++;
+                }
                 push @code, 'my $arg_err; ' unless keys %vargs;
                 $vargs{$arg} = 1;
                 push @code, __squish_code($cd->{result}), "; ";
@@ -211,6 +234,8 @@ sub munge_file {
             $sub_name      = $1;
             $sub_has_vargs = 0;
             %vargs         = ();
+            @modules       = ();
+            %vars          = ();
             $meta          = $metas->{$sub_name};
             next;
         }
@@ -309,7 +334,7 @@ Dist::Zilla::Plugin::Rinci::Validate - Insert argument validator code in output
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -395,6 +420,8 @@ or:
  sub foo {
      # NO_VALIDATE_ARGS
 
+=for Pod::Coverage ^(munge_file|munge_files)$
+
 =head1 FAQ
 
 =head2 Rationale for this plugin?
@@ -446,7 +473,7 @@ Steven Haryanto <stevenharyanto@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2012 by Steven Haryanto.
+This software is copyright (c) 2013 by Steven Haryanto.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
